@@ -5,7 +5,7 @@ use strict;
 use Mouse;
 use JSON qw(from_json to_json);
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.2';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin',
   'Lemonldap::NG::Portal::Lib::U2F';
@@ -29,6 +29,11 @@ sub init {
 # Main method
 sub run {
     my ( $self, $req, $action ) = @_;
+    my $user = $req->userData->{ $self->conf->{whatToTrace} };
+    unless ($user) {
+        return $self->p->sendError( $req,
+            'No ' . $self->conf->{whatToTrace} . ' found in user data', 500 );
+    }
 
     if ( $action eq 'register' ) {
 
@@ -142,6 +147,8 @@ sub run {
                     "Append 2F Device : { type => 'U2F', name => $keyName }");
                 $self->p->updatePersistentSession( $req,
                     { _2fDevices => to_json($_2fDevices) } );
+                $self->userLogger->notice(
+                    "U2F key registration of $keyName succeeds for $user");
 
                 return [
                     200,
@@ -186,8 +193,7 @@ sub run {
         }
 
         # Serialize data
-        $data = to_json(
-            {
+        $data = to_json( {
                 challenge      => $data->{challenge},
                 appId          => $data->{appId},
                 registeredKeys => \@rk
@@ -298,9 +304,6 @@ sub run {
             [ 'Content-Type' => 'application/json', 'Content-Length' => 12, ],
             ['{"result":1}']
         ];
-        my $err = Crypt::U2F::Server::Simple::lastError();
-        $self->userLogger->warn("U2F Unregistration failed: $err");
-        return $self->p->sendError( $req, $err, 200 );
     }
     else {
         $self->logger->error("Unknown U2F action -> $action");
@@ -316,7 +319,6 @@ sub loadUser {
     # Read existing 2FDevices
     $self->logger->debug("Looking for 2F Devices ...");
     my ( $kh, $uk, $_2fDevices );
-
     my @u2fs = ();
 
     if ( $req->userData->{_2fDevices} ) {
