@@ -3,36 +3,55 @@ package Lemonldap::NG::Portal::Issuer::CAS;
 use strict;
 use Mouse;
 use URI;
+use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_CAS_SERVICE_NOT_ALLOWED
   PE_CONFIRM
   PE_ERROR
   PE_LOGOUT_OK
   PE_OK
+  PE_SENDRESPONSE
 );
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.3';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::CAS';
 
-# INTERFACE
-
-sub beforeAuth { 'exportRequestParameters' }
-
 # INITIALIZATION
 
+use constant beforeAuth  => 'storeEnvAndCheckGateway';
+use constant sessionKind => 'ICAS';
+
+has rule => ( is => 'rw' );
+
 sub init {
     my ($self) = @_;
 
-    # Launch parents initialization subroutines, then launch IdP en SP lists
+    # Parse activation rule
+    my $hd = $self->p->HANDLER;
+    $self->logger->debug( "CAS rule -> " . $self->conf->{issuerDBCASRule} );
+    my $rule =
+      $hd->buildSub( $hd->substitute( $self->conf->{issuerDBCASRule} ) );
+    unless ($rule) {
+        $self->error( "Bad CAS rule -> " . $hd->tsv->{jail}->error );
+        return 0;
+    }
+    $self->{rule} = $rule;
+
+    # Launch parents initialization subroutines, then launch IdP and SP lists
     my $res = $self->Lemonldap::NG::Portal::Main::Issuer::init();
+    return 0 unless ( $self->loadApp );
     $self->addUnauthRoute(
         ( $self->path ) => {
             serviceValidate => 'serviceValidate',
             validate        => 'validate',
             proxyValidate   => 'proxyValidate',
-            proxy           => 'proxy'
+            proxy           => 'proxy',
+            p3              => {
+                serviceValidate => 'serviceValidate',
+                proxyValidate   => 'proxyValidate'
+            }
         },
         ['GET']
     );
@@ -41,17 +60,52 @@ sub init {
 
 # RUNNING METHODS
 
+sub storeEnvAndCheckGateway {
+    my ( $self, $req ) = @_;
+    my $service = $self->p->getHiddenFormValue( $req, 'service' )
+      || $req->param('service');
+    my $gateway = $self->p->getHiddenFormValue( $req, 'gateway' )
+      || $req->param('gateway');
+
+    if ( $gateway and $gateway eq "true" ) {
+        $self->logger->debug(
+            "Gateway mode requested, redirect without authentication");
+        $req->response( [ 302, [ Location => $service ], [] ] );
+        $req->pdata( {} );
+        return PE_SENDRESPONSE;
+    }
+
+    if ( $service and $service =~ m#^(https?://[^/]+)(/.*)?$# ) {
+        my ( $host, $uri ) = ( $1, $2 );
+        my $app = $self->casAppList->{$host};
+
+        if ($app) {
+            $req->env->{llng_cas_app} = $app;
+        }
+    }
+
+    return PE_OK;
+}
+
 # Main method (launched only for authenticated users, see Main/Issuer)
 sub run {
     my ( $self, $req, $target ) = @_;
 
+    # Check activation rule
+    unless ( $self->rule->( $req, $req->sessionInfo ) ) {
+        $self->userLogger->error('CAS service not authorized');
+        return PE_CAS_SERVICE_NOT_ALLOWED;
+    }
+
     # CAS URL
-    my $cas_login           = 'login';
-    my $cas_logout          = 'logout';
-    my $cas_validate        = 'validate';
-    my $cas_serviceValidate = 'serviceValidate';
-    my $cas_proxyValidate   = 'proxyValidate';
-    my $cas_proxy           = 'proxy';
+    my $cas_login              = 'login';
+    my $cas_logout             = 'logout';
+    my $cas_validate           = 'validate';
+    my $cas_serviceValidate    = 'serviceValidate';
+    my $cas_p3_serviceValidate = 'p3/serviceValidate';
+    my $cas_proxyValidate      = 'proxyValidate';
+    my $cas_p3_proxyValidate   = 'p3/proxyValidate';
+    my $cas_proxy              = 'proxy';
 
     # Called URL
     my $url = $req->uri();
@@ -70,28 +124,24 @@ sub run {
         # GET parameters
         my $service = $self->p->getHiddenFormValue( $req, 'service' )
           || $req->param('service');
-        my $renew =
-          $self->p->getHiddenFormValue( $req, 'renew' ) || $req->param('renew');
+        my $renew = $self->p->getHiddenFormValue( $req, 'renew' )
+          || $req->param('renew');
         my $gateway = $self->p->getHiddenFormValue( $req, 'gateway' )
           || $req->param('gateway');
         my $casServiceTicket;
 
         # Renew
-        if ( $renew and $renew eq 'true' ) {
+        if (    $renew
+            and $renew eq 'true'
+            and time - $req->sessionInfo->{_utime} >
+            $self->conf->{portalForceAuthnInterval} )
+        {
 
             # Authentication must be replayed
             $self->logger->debug("Authentication renew requested");
             $self->{updateSession} = 1;
-            $req->steps(
-                [
-                    @{ $self->p->beforeAuth },
-                    $self->p->authProcess,
-                    @{ $self->p->betweenAuthAndDatas },
-                    $self->p->sessionDatas,
-                    @{ $self->p->afterDatas },
-                ]
-            );
-            return PE_OK;
+            $req->env->{QUERY_STRING} =~ s/renew=true/renew=false/;
+            return $self->reAuth($req);
         }
 
         # If no service defined, exit
@@ -100,6 +150,13 @@ sub run {
             return PE_OK;
         }
 
+        unless ( $service =~ m#^(https?://[^/]+)(/.*)?$# ) {
+            $self->logger->error("Bad service $service");
+            return PE_ERROR;
+        }
+        my ( $host, $uri ) = ( $1, $2 );
+        my $app = $self->casAppList->{$host};
+
         # Check access on the service
         my $casAccessControlPolicy = $self->conf->{casAccessControlPolicy};
 
@@ -107,32 +164,31 @@ sub run {
             $self->logger->debug(
                 "CAS access control requested on service $service");
 
-            ## HERE
-            unless ( $service =~ m#^https?://([^/]+)(/.*)?$# ) {
-                $self->logger->error("Bad service $service");
-                return PE_ERROR;
+            unless ($app) {
+                $self->userLogger->error('CAS service not configured');
+                return PE_CAS_SERVICE_NOT_ALLOWED;
             }
-            my ( $host, $uri ) = ( $1, $2 );
-            if ( $self->p->HANDLER->grant( $req->sessionInfo, $1, undef, $2 ) )
-            {
-                $self->logger->debug("CAS service $service access allowed");
-            }
-
-            else {
-                $self->userLogger->error(
-                    "CAS service $service access not allowed");
-
-                if ( $casAccessControlPolicy =~ /^(error)$/i ) {
-                    $self->logger->debug(
-"Return error instead of redirecting user on CAS service"
-                    );
-                    return PE_CAS_SERVICE_NOT_ALLOWED;
+            if ( my $rule = $self->spRules->{$app} ) {
+                if ( $rule->( $req, $req->sessionInfo ) ) {
+                    $self->logger->debug("CAS service $service access allowed");
                 }
 
                 else {
-                    $self->logger->debug(
-                        "Redirect user on CAS service with a fake ticket");
-                    $casServiceTicket = "ST-F4K3T1CK3T";
+                    $self->userLogger->error(
+                        "CAS service $service access not allowed");
+
+                    if ( $casAccessControlPolicy =~ /^(error)$/i ) {
+                        $self->logger->debug(
+"Return error instead of redirecting user on CAS service"
+                        );
+                        return PE_CAS_SERVICE_NOT_ALLOWED;
+                    }
+
+                    else {
+                        $self->logger->debug(
+                            "Redirect user on CAS service with a fake ticket");
+                        $casServiceTicket = "ST-F4K3T1CK3T";
+                    }
                 }
             }
         }
@@ -163,6 +219,7 @@ sub run {
             $Sinfos->{renew}   = $casRenewFlag;
             $Sinfos->{_cas_id} = $session_id;
             $Sinfos->{_utime}  = $time;
+            $Sinfos->{_casApp} = $app;
 
             my $casServiceSession = $self->getCasSession( undef, $Sinfos );
 
@@ -180,11 +237,8 @@ sub run {
 
         # Redirect to service
         my $service_url = $service;
-        $service_url .= (
-            $service =~ /\?/
-            ? '&ticket=' . $casServiceTicket
-            : '?ticket=' . $casServiceTicket
-        );
+        $service_url .= ( $service =~ /\?/ ? '&' : '?' )
+          . build_urlencoded( ticket => $casServiceTicket );
 
         $self->logger->debug("Redirect user to $service_url");
 
@@ -204,7 +258,8 @@ sub run {
         $req->frame(1);
 
         # GET parameters
-        my $logout_url = $req->param('url');
+        my $logout_url     = $req->param('url');        # CAS 2.0
+        my $logout_service = $req->param('service');    # CAS 3.0
 
         # Delete linked CAS sessions
         $self->deleteCasSecondarySessions($session_id);
@@ -222,13 +277,22 @@ sub run {
                     "Logout URL $logout_url will be displayed");
 
                 $req->info(
-'<h3 trmsg="back2CasUrl">The application you just logged out of has provided a link it would like you to follow</h3>'
+                    $self->loadTemplate(
+                        'casBack2Url', params => { url => $logout_url }
+                    )
                 );
-                $req->info("<p><a href=\"$logout_url\">$logout_url</a></p>");
-                $req->datas->{activeTimer} = 0;
+                $req->data->{activeTimer} = 0;
 
                 return PE_CONFIRM;
             }
+
+            if ($logout_service) {
+                $self->logger->debug(
+                    "User will be redirected to $logout_service");
+                $req->{urldc} = $logout_service;
+                $req->steps( [] );
+                return PE_OK;
+            }
         }
         else {
             $self->logger->info("Unknown session $session_id");
@@ -251,7 +315,9 @@ sub run {
     }
 
     # 4. SERVICE VALIDATE [CAS 2.0]
-    if ( $target eq $cas_serviceValidate ) {
+    if (   $target eq $cas_serviceValidate
+        || $target eq $cas_p3_serviceValidate )
+    {
 
         $self->logger->debug(
             "URL $url detected as an CAS SERVICE VALIDATE URL");
@@ -264,7 +330,7 @@ sub run {
     }
 
     # 5. PROXY VALIDATE [CAS 2.0]
-    if ( $target eq $cas_proxyValidate ) {
+    if ( $target eq $cas_proxyValidate || $target eq $cas_p3_proxyValidate ) {
 
         $self->logger->debug("URL $url detected as an CAS PROXY VALIDATE URL");
 
@@ -297,7 +363,7 @@ sub logout {
     my $session_id = $req->{sessionInfo}->{_session_id} || $req->{id};
 
     # Delete linked CAS sessions
-    $self->deleteCasSecondarySessions($session_id);
+    $self->deleteCasSecondarySessions($session_id) if ($session_id);
 
     return PE_OK;
 }
@@ -392,8 +458,7 @@ sub validate {
     }
 
     # Get username
-    my $username =
-      $localSession->data->{ $self->conf->{casAttr}
+    my $username = $localSession->data->{ $self->conf->{casAttr}
           || $self->conf->{whatToTrace} };
 
     $self->logger->debug("Get username $username");
@@ -405,6 +470,68 @@ sub validate {
 
 sub proxy {
     my ( $self, $req ) = @_;
+
+    $self->logger->debug(
+        'URL ' . $req->uri . " detected as an CAS PROXY URL" );
+
+    # GET parameters
+    my $pgt           = $req->param('pgt');
+    my $targetService = $req->param('targetService');
+
+    # Required parameters: pgt and targetService
+    unless ( $pgt and $targetService ) {
+        $self->logger->error("Pgt and TargetService parameters required");
+        $self->returnCasProxyError( $req, 'INVALID_REQUEST',
+            'Missing mandatory parameters (pgt, targetService)' );
+    }
+
+    $self->logger->debug(
+        "Get proxy request with ticket $pgt for service $targetService");
+
+    # Get CAS session corresponding to ticket
+    unless ( $pgt =~ s/^PGT-// ) {
+        $self->logger->error(
+            "Provided ticket is not a proxy granting ticket (PGT)");
+        $self->returnCasProxyError( $req, 'BAD_PGT',
+            'Provided ticket is not a proxy granting ticket' );
+    }
+
+    my $casProxyGrantingSession = $self->getCasSession($pgt);
+
+    unless ($casProxyGrantingSession) {
+        $self->logger->error("Proxy granting ticket session $pgt not found");
+        $self->returnCasProxyError( $req, 'BAD_PGT', 'Ticket not found' );
+    }
+
+    $self->logger->debug("Proxy granting session $pgt found");
+
+    # Create a proxy ticket
+    $self->logger->debug(
+        "Create a CAS proxy ticket for service $targetService");
+
+    my $casProxySession = $self->getCasSession();
+
+    unless ($casProxySession) {
+        $self->logger->error("Unable to create CAS proxy session");
+        $self->returnCasProxyError( $req, 'INTERNAL_ERROR',
+            'Error in proxy session management' );
+    }
+
+    my $Pinfos;
+    $Pinfos->{type}    = 'casProxy';
+    $Pinfos->{service} = $targetService;
+    $Pinfos->{_cas_id} = $casProxyGrantingSession->data->{_cas_id};
+    $Pinfos->{_utime}  = $casProxyGrantingSession->data->{_utime};
+    $Pinfos->{proxies} = $casProxyGrantingSession->data->{proxies};
+
+    $casProxySession->update($Pinfos);
+
+    my $casProxySessionID = $casProxySession->id;
+    my $casProxyTicket    = "PT-" . $casProxySessionID;
+
+    $self->logger->debug("CAS proxy session $casProxySessionID created");
+
+    return $self->returnCasProxySuccess( $req, $casProxyTicket );
 }
 
 sub serviceValidate {
@@ -436,7 +563,7 @@ sub _validate2 {
     # Required parameters: service and ticket
     unless ( $service and $ticket ) {
         $self->logger->error("Service and Ticket parameters required");
-        return $self->returnCasServiceValidateError( 'INVALID_REQUEST',
+        return $self->returnCasServiceValidateError( $req, 'INVALID_REQUEST',
             'Missing mandatory parameters (service, ticket)' );
     }
 
@@ -447,13 +574,13 @@ sub _validate2 {
     # Get CAS session corresponding to ticket
     if ( $urlType eq 'SERVICE' and !( $ticket =~ s/^ST-// ) ) {
         $self->logger->error("Provided ticket is not a service ticket (ST)");
-        return $self->returnCasServiceValidateError( 'INVALID_TICKET',
+        return $self->returnCasServiceValidateError( $req, 'INVALID_TICKET',
             'Provided ticket is not a service ticket' );
     }
     elsif ( $urlType eq 'PROXY' and !( $ticket =~ s/^(P|S)T-// ) ) {
         $self->userLogger->error(
             "Provided ticket is not a service or proxy ticket ($1T)");
-        return $self->returnCasServiceValidateError( 'INVALID_TICKET',
+        return $self->returnCasServiceValidateError( $req, 'INVALID_TICKET',
             'Provided ticket is not a service or proxy ticket' );
     }
 
@@ -461,9 +588,10 @@ sub _validate2 {
 
     unless ($casServiceSession) {
         $self->logger->error("$urlType ticket session $ticket not found");
-        return $self->returnCasServiceValidateError( 'INVALID_TICKET',
+        return $self->returnCasServiceValidateError( $req, 'INVALID_TICKET',
             'Ticket not found' );
     }
+    my $app = $casServiceSession->data->{_casApp};
 
     $self->logger->debug("$urlType ticket session $ticket found");
 
@@ -487,7 +615,8 @@ sub _validate2 {
                 "Submitted service $service does not match initial service "
                   . $casServiceSession->data->{service} );
             $self->deleteCasSession($casServiceSession);
-            return $self->returnCasServiceValidateError( 'INVALID_SERVICE',
+            return $self->returnCasServiceValidateError( $req,
+                'INVALID_SERVICE',
                 'Submitted service does not match initial service' );
         }
     }
@@ -529,11 +658,12 @@ sub _validate2 {
         $PGinfos->{service} = $service;
         $PGinfos->{_cas_id} = $casServiceSession->data->{_cas_id};
         $PGinfos->{_utime}  = $casServiceSession->data->{_utime};
+        $PGinfos->{_casApp} = $app;
 
         # Trace proxies
         $PGinfos->{proxies} = (
               $proxies
-            ? $proxies . $self->{multiValuesSeparator} . $pgtUrl
+            ? $proxies . $self->conf->{multiValuesSeparator} . $pgtUrl
             : $pgtUrl
         );
 
@@ -593,29 +723,30 @@ sub _validate2 {
               . $casServiceSession->data->{_cas_id}
               . " notfound" );
         $self->deleteCasSession($casServiceSession);
-        return $self->returnCasServiceValidateError( 'INTERNAL_ERROR',
+        return $self->returnCasServiceValidateError( $req, 'INTERNAL_ERROR',
             'No session associated to ticket' );
     }
 
     # Get username
-    my $username =
-      $localSession->data->{ $self->conf->{casAttr}
+    my $username = $localSession->data->{ $self->conf->{casAttr}
           || $self->conf->{whatToTrace} };
 
     $self->logger->debug("Get username $username");
 
     # Get attributes [CAS 3.0]
     my $attributes = {};
-    if ( defined $self->conf->{casAttributes}
-        and %{ $self->conf->{casAttributes} } )
-    {
-        foreach my $casAttribute ( keys %{ $self->conf->{casAttributes} } ) {
-            my $localSessionValue =
-              $localSession->data->{ $self->conf->{casAttributes}
-                  ->{$casAttribute} };
-            $attributes->{$casAttribute} = $localSessionValue
-              if defined $localSessionValue;
-        }
+    my $ev =
+      ( $app and $self->conf->{casAppMetaDataExportedVars}->{$app} )
+      ? $self->conf->{casAppMetaDataExportedVars}->{$app}
+      : {};
+    unless (%$ev) {
+        $ev = $self->conf->{casAttributes} || {};
+    }
+
+    foreach my $casAttribute ( keys %$ev ) {
+        my $localSessionValue = $localSession->data->{ $ev->{$casAttribute} };
+        $attributes->{$casAttribute} = $localSessionValue
+          if defined $localSessionValue;
     }
 
     # Return success message
@@ -624,17 +755,4 @@ sub _validate2 {
         $casProxyGrantingTicketIOU, $proxies, $attributes );
 }
 
-# Store request parameters in %ENV
-sub exportRequestParameters {
-    my ( $self, $req ) = @_;
-
-    foreach my $param (qw/service renew gateway/) {
-        if ( $req->param($param) ) {
-            $req->env->{ "llng_cas_" . $param } = $req->param($param);
-        }
-    }
-
-    return PE_OK;
-}
-
 1;
