@@ -12,9 +12,10 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_REDIRECT
   PE_OK
   PE_UNAUTHORIZEDPARTNER
+  PE_OIDC_SERVICE_NOT_ALLOWED
 );
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.3';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::OpenIDConnect',
@@ -28,6 +29,7 @@ sub beforeAuth { 'exportRequestParameters' }
 
 use constant sessionKind => 'OIDCI';
 
+has rule => ( is => 'rw' );
 has configStorage => (
     is      => 'ro',
     lazy    => 1,
@@ -35,7 +37,6 @@ has configStorage => (
         $_[0]->{p}->HANDLER->localConfig->{configStorage};
     }
 );
-
 has ssoMatchUrl => ( is => 'rw' );
 
 # OIDC has 7 endpoints managed here as PSGI endpoints or in run() [Main/Issuer.pm
@@ -56,6 +57,19 @@ has ssoMatchUrl => ( is => 'rw' );
 sub init {
     my ($self) = @_;
 
+    # Parse activation rule
+    my $hd = $self->p->HANDLER;
+    $self->logger->debug(
+        "OIDC rule -> " . $self->conf->{issuerDBOpenIDConnectRule} );
+    my $rule =
+      $hd->buildSub(
+        $hd->substitute( $self->conf->{issuerDBOpenIDConnectRule} ) );
+    unless ($rule) {
+        $self->error( "Bad OIDC rule -> " . $hd->tsv->{jail}->error );
+        return 0;
+    }
+    $self->{rule} = $rule;
+
     # Initialize RP list
     return 0
       unless ( $self->Lemonldap::NG::Portal::Main::Issuer::init()
@@ -113,6 +127,13 @@ sub ssoMatch {
 # run() manages only "authorize" and "logout" endpoints.
 sub run {
     my ( $self, $req, $path ) = @_;
+
+    # Check activation rule
+    unless ( $self->rule->( $req, $req->sessionInfo ) ) {
+        $self->userLogger->error('OIDC service not authorized');
+        return PE_OIDC_SERVICE_NOT_ALLOWED;
+    }
+
     if ($path) {
 
         # Convert old format OIDC Consents
@@ -177,8 +198,8 @@ sub run {
 "Override $_ OIDC param by value present in request parameter"
                     );
                     $oidc_request->{$_} = $request->{$_};
-                    $self->p->setHiddenFormValue( $req, $_, $request->{$_},
-                        '' );
+                    $self->p->setHiddenFormValue( $req, $_, $request->{$_}, '',
+                        0 );
                 }
             }
 
@@ -306,6 +327,13 @@ sub run {
                 return $self->reAuth($req);
             }
 
+            # Check scope validity
+            unless ( $oidc_request->{'scope'} =~ /^[a-zA-Z_\-\s]+$/ ) {
+                $self->logger->error( "Submitted scope is not valid: "
+                      . $oidc_request->{'scope'} );
+                return PE_ERROR;
+            }
+
             # Check openid scope
             unless ( $oidc_request->{'scope'} =~ /\bopenid\b/ ) {
                 $self->logger->debug("No openid scope found");
@@ -827,7 +855,8 @@ sub run {
                     $self->logger->debug( "OIDC request parameter $param: "
                           . $oidc_request->{$param} );
                     $self->p->setHiddenFormValue( $req, $param,
-                        $oidc_request->{$param}, '' );
+                        $oidc_request->{$param},
+                        '', 0 );
                 }
             }
 
@@ -839,8 +868,7 @@ sub run {
             if ( $req->param('confirm') ) {
                 my $err;
                 if ( $req->param('confirm') == 1 ) {
-                    $req->steps(
-                        [
+                    $req->steps( [
                             @{ $self->p->beforeLogout }, 'authLogout',
                             'deleteSession'
                         ]
