@@ -12,7 +12,7 @@ use Unicode::String qw(utf8);
 use Scalar::Util 'weaken';
 use utf8;
 
-our $VERSION  = '2.0.0';
+our $VERSION  = '2.0.3';
 our $ppLoaded = 0;
 
 BEGIN {
@@ -190,7 +190,7 @@ sub userBind {
         my $pp_error = $resp->pp_error;
         if ( defined $pp_error ) {
             $self->{portal}->userLogger->error(
-                "Password policy error $pp_error for $self->{portal}->{user}");
+                "Password policy error $pp_error for " . $req->user );
             return [
                 PE_PP_PASSWORD_EXPIRED,
                 PE_PP_ACCOUNT_LOCKED,
@@ -207,10 +207,8 @@ sub userBind {
 
             # Get expiration warning and graces
             if ( $resp->grace_authentications_remaining ) {
-
-                # TODO
-                $self->info(
-                    $self->loadTemplate(
+                $req->info(
+                    $self->{portal}->loadTemplate(
                         'ldapPpGrace',
                         params => {
                             number => $resp->grace_authentications_remaining
@@ -221,7 +219,7 @@ sub userBind {
 
             if ( $resp->time_before_expiration ) {
                 $req->info(
-                    $self->loadTemplate(
+                    $self->{portal}->loadTemplate(
                         'simpleInfo',
                         params => {
                             trspan => 'authRemaining,'
