@@ -19,7 +19,7 @@ use Mouse;
 
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_REDIRECT);
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.3';
 
 # OpenID Connect standard claims
 use constant PROFILE => [
@@ -564,7 +564,7 @@ sub checkIDTokenValidity {
             $self->logger->error("Auth time was not returned by OP $op");
             return 0;
         }
-        if ( $auth_time + $max_age > time ) {
+        if ( time > $auth_time + $max_age ) {
             $self->userLogger->error(
 "Authentication time ($auth_time) is too old (Max age: $max_age)"
             );
@@ -644,8 +644,7 @@ sub getOpenIDConnectSession {
         );
     }
 
-    my $oidcSession = Lemonldap::NG::Common::Session->new(
-        {
+    my $oidcSession = Lemonldap::NG::Common::Session->new( {
             %storage,
             cacheModule        => $self->conf->{localSessionStorage},
             cacheModuleOptions => $self->conf->{localSessionStorageOptions},
