@@ -2,10 +2,13 @@
 # Display functions for LemonLDAP::NG Portal
 package Lemonldap::NG::Portal::Main::Display;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.3';
 
 package Lemonldap::NG::Portal::Main;
 use strict;
+use Mouse;
+use JSON;
+use Data::Dumper;
 
 has skinRules => ( is => 'rw' );
 
@@ -36,78 +39,113 @@ sub display {
     my $skin_dir = $self->conf->{templatesDir};
     my ( $skinfile, %templateParams );
 
-    # 0. Display error page
-    if ( my $http_error = $req->param('lmError') ) {
-
-        $skinfile = 'error';
-
-        # Check URL
-        $self->controlUrl($req);
-
-        %templateParams = (
-            LOGOUT_URL => $self->conf->{portal} . "?logout=1",
-            URL        => $req->{urldc},
-        );
-
-        # Error code
-        foreach ( 403, 500, 503 ) {
-            $templateParams{"ERROR$_"} = ( $http_error == $_ ? 1 : 0 );
-        }
-
-    }
-
     # 1. Authentication not complete
 
     # 1.1 A notification has to be done (session is created but hidden and
     #     unusable until the user has accept the message)
-    if ( my $notif = $req->datas->{notification} ) {
+    if ( my $notif = $req->data->{notification} ) {
+        $self->logger->debug('Display: notification detected');
         $skinfile       = 'notification';
         %templateParams = (
+            MAIN_LOGO       => $self->conf->{portalMainLogo},
+            LANGS           => $self->conf->{showLanguages},
             AUTH_ERROR_TYPE => $req->error_type,
             NOTIFICATION    => $notif,
             HIDDEN_INPUTS   => $self->buildHiddenForm($req),
-            AUTH_URL        => $req->{datas}->{_url},
+            AUTH_URL        => $req->{data}->{_url},
             CHOICE_PARAM    => $self->conf->{authChoiceParam},
-            CHOICE_VALUE    => $req->datas->{_authChoice},
+            CHOICE_VALUE    => $req->data->{_authChoice},
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
         );
     }
 
-    # 1.2 An authentication (or userDB) module needs to ask a question
+    # 1.2a An authentication (or userDB) module needs to ask a question
     #     before processing to the request
     elsif ( $req->{error} == PE_CONFIRM ) {
+        $self->logger->debug('Display: confirm detected');
         $skinfile       = 'confirm';
         %templateParams = (
+            MAIN_LOGO       => $self->conf->{portalMainLogo},
+            LANGS           => $self->conf->{showLanguages},
             AUTH_ERROR      => $req->error,
             AUTH_ERROR_TYPE => $req->error_type,
-            AUTH_URL        => $req->{datas}->{_url},
+            AUTH_URL        => $req->{data}->{_url},
             MSG             => $req->info,
             HIDDEN_INPUTS   => $self->buildHiddenForm($req),
-            ACTIVE_TIMER    => $req->datas->{activeTimer},
+            ACTIVE_TIMER    => $req->data->{activeTimer},
             FORM_METHOD     => $self->conf->{confirmFormMethod},
             CHOICE_PARAM    => $self->conf->{authChoiceParam},
-            CHOICE_VALUE    => $req->datas->{_authChoice},
+            CHOICE_VALUE    => $req->data->{_authChoice},
             CHECK_LOGINS    => $self->conf->{portalCheckLogins}
-              && $req->datas->{login},
+              && $req->data->{login},
             ASK_LOGINS => $req->param('checkLogins') || 0,
             CONFIRMKEY => $self->stamp(),
-            LIST       => $req->datas->{list}        || [],
-            REMEMBER   => $req->datas->{confirmRemember},
+            REMEMBER   => $req->data->{confirmRemember},
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
+        );
+    }
+
+    # 1.2b An authentication (or userDB) module needs to ask a question
+    #     before processing to the request
+    elsif ( $req->{error} == PE_IDPCHOICE ) {
+        $self->logger->debug('Display: IDP choice detected');
+        $skinfile       = 'idpchoice';
+        %templateParams = (
+            MAIN_LOGO       => $self->conf->{portalMainLogo},
+            LANGS           => $self->conf->{showLanguages},
+            AUTH_ERROR      => $req->error,
+            AUTH_ERROR_TYPE => $req->error_type,
+            AUTH_URL        => $req->{data}->{_url},
+            HIDDEN_INPUTS   => $self->buildHiddenForm($req),
+            ACTIVE_TIMER    => $req->data->{activeTimer},
+            FORM_METHOD     => $self->conf->{confirmFormMethod},
+            CHOICE_PARAM    => $self->conf->{authChoiceParam},
+            CHOICE_VALUE    => $req->data->{_authChoice},
+            CHECK_LOGINS    => $self->conf->{portalCheckLogins}
+              && $req->data->{login},
+            ASK_LOGINS => $req->param('checkLogins') || 0,
+            CONFIRMKEY => $self->stamp(),
+            LIST       => $req->data->{list}         || [],
+            REMEMBER   => $req->data->{confirmRemember},
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
         );
     }
 
     # 1.3 There is a message to display
     elsif ( my $info = $req->info ) {
+        $self->logger->debug('Display: info detected');
+        $self->logger->debug(
+            'Hidden values -> ' . Dumper( $req->{portalHiddenFormValues} ) );
         $skinfile       = 'info';
         %templateParams = (
+            MAIN_LOGO       => $self->conf->{portalMainLogo},
+            LANGS           => $self->conf->{showLanguages},
             AUTH_ERROR      => $self->error,
             AUTH_ERROR_TYPE => $req->error_type,
             MSG             => $info,
             URL             => $req->{urldc},
             HIDDEN_INPUTS   => $self->buildHiddenForm($req),
-            ACTIVE_TIMER    => $req->datas->{activeTimer},
+            ACTIVE_TIMER    => $req->data->{activeTimer},
             FORM_METHOD     => $self->conf->{infoFormMethod},
             CHOICE_PARAM    => $self->conf->{authChoiceParam},
-            CHOICE_VALUE    => $req->datas->{_authChoice},
+            CHOICE_VALUE    => $req->data->{_authChoice},
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
         );
     }
 
@@ -121,12 +159,19 @@ sub display {
         my $id = $req->{sessionInfo}
           ->{ $self->conf->{openIdAttr} || $self->conf->{whatToTrace} };
         %templateParams = (
+            MAIN_LOGO       => $self->conf->{portalMainLogo},
+            LANGS           => $self->conf->{showLanguages},
             AUTH_ERROR      => $self->error,
             AUTH_ERROR_TYPE => $req->error_type,
             PROVIDERURI     => $p,
             MSG             => $req->info(),
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
         );
-        $templateParams{ID} = $req->datas->{_openidPortal} . $id if ($id);
+        $templateParams{ID} = $req->data->{_openidPortal} . $id if ($id);
     }
 
     # 2. Good authentication
@@ -137,7 +182,12 @@ sub display {
         %templateParams = (
             URL           => $req->{urldc},
             HIDDEN_INPUTS => $self->buildHiddenForm($req),
-            FORM_METHOD   => $req->datas->{redirectFormMethod} || 'get',
+            FORM_METHOD   => $req->data->{redirectFormMethod} || 'get',
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
         );
     }
 
@@ -147,21 +197,77 @@ sub display {
 
         #utf8::decode($auth_user);
         %templateParams = (
+            MAIN_LOGO => $self->conf->{portalMainLogo},
+            LANGS     => $self->conf->{showLanguages},
             AUTH_USER => $req->{sessionInfo}->{ $self->conf->{portalUserAttr} },
             NEWWINDOW => $self->conf->{portalOpenLinkInNewWindow},
-            LOGOUT_URL     => $self->conf->{portal} . "?logout=1",
-            APPSLIST_ORDER => $req->{sessionInfo}->{'appsListOrder'},
-            PING           => $self->conf->{portalPingInterval},
+            LOGOUT_URL          => $self->conf->{portal} . "?logout=1",
+            APPSLIST_ORDER      => $req->{sessionInfo}->{'_appsListOrder'},
+            PING                => $self->conf->{portalPingInterval},
+            REQUIRE_OLDPASSWORD => $self->conf->{portalRequireOldPassword},
+            HIDE_OLDPASSWORD    => 0,
             $self->menu->params($req),
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
         );
     }
 
-    # 2.3 Case : user authenticated but an error was returned (bas url,...)
-    elsif ( $req->userData and %{ $req->userData } ) {
+    elsif ( $req->error == PE_RENEWSESSION ) {
+        $skinfile       = 'upgradesession';
+        %templateParams = (
+            MAIN_LOGO  => $self->conf->{portalMainLogo},
+            LANGS      => $self->conf->{showLanguages},
+            MSG        => 'askToRenew',
+            CONFIRMKEY => $self->stamp,
+            PORTAL     => $self->conf->{portal},
+            URL        => $req->data->{_url},
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
+        );
+    }
+
+    elsif ( $req->error == PE_MUSTAUTHN ) {
+        $skinfile       = 'updatesession';
+        %templateParams = (
+            MAIN_LOGO  => $self->conf->{portalMainLogo},
+            LANGS      => $self->conf->{showLanguages},
+            MSG        => 'PE87',
+            CONFIRMKEY => $self->stamp,
+            PORTAL     => $self->conf->{portal},
+            URL        => $req->data->{_url},
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
+        );
+    }
+
+    # 2.3 Case : user authenticated but an error was returned (bad url,...)
+    elsif (
+        $req->noLoginDisplay
+        or (    not $req->data->{noerror}
+            and $req->userData
+            and %{ $req->userData } )
+      )
+    {
         $skinfile       = 'error';
         %templateParams = (
+            MAIN_LOGO       => $self->conf->{portalMainLogo},
+            LANGS           => $self->conf->{showLanguages},
             AUTH_ERROR      => $req->error,
             AUTH_ERROR_TYPE => $req->error_type,
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
         );
     }
 
@@ -171,9 +277,11 @@ sub display {
         my $login = $self->userId($req);
         $login = '' if ( $login eq 'anonymous' );
         %templateParams = (
+            MAIN_LOGO             => $self->conf->{portalMainLogo},
+            LANGS                 => $self->conf->{showLanguages},
             AUTH_ERROR            => $req->error,
             AUTH_ERROR_TYPE       => $req->error_type,
-            AUTH_URL              => $req->{datas}->{_url},
+            AUTH_URL              => $req->{data}->{_url},
             LOGIN                 => $login,
             CHECK_LOGINS          => $self->conf->{portalCheckLogins},
             ASK_LOGINS            => $req->param('checkLogins') || 0,
@@ -183,6 +291,12 @@ sub display {
             REGISTER_URL          => $self->conf->{registerUrl},
             HIDDEN_INPUTS         => $self->buildHiddenForm($req),
             STAYCONNECTED         => $self->conf->{stayConnected},
+            SPOOFID               => $self->conf->{impersonationRule},
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
         );
 
         # Display captcha if it's enabled
@@ -224,11 +338,11 @@ sub display {
                 DISPLAY_RESETPASSWORD => 0,
                 AUTH_LOOP             => [],
                 CHOICE_PARAM          => $self->conf->{authChoiceParam},
-                CHOICE_VALUE          => $req->datas->{_authChoice},
+                CHOICE_VALUE          => $req->data->{_authChoice},
                 OLDPASSWORD           => $self->checkXSSAttack( 'oldpassword',
-                    $req->datas->{oldpassword} )
+                    $req->data->{oldpassword} )
                 ? ""
-                : $req->datas->{oldpassword},
+                : $req->data->{oldpassword},
                 HIDE_OLDPASSWORD => $self->conf->{hideOldPassword},
             );
         }
@@ -237,6 +351,7 @@ sub display {
         # * Logout message
         # * Bad URL error
         elsif ($req->{error} == PE_LOGOUT_OK
+            or $req->{error} == PE_WAIT
             or $req->{error} == PE_BADURL )
         {
             %templateParams = (
@@ -255,14 +370,14 @@ sub display {
         else {
 
             # Authentication loop
-            if ( $req->sessionInfo->{_choice}
+            if ( $self->conf->{authentication} eq 'Choice'
                 and my $authLoop = $self->_buildAuthLoop($req) )
             {
                 %templateParams = (
                     %templateParams,
                     AUTH_LOOP            => $authLoop,
                     CHOICE_PARAM         => $self->conf->{authChoiceParam},
-                    CHOICE_VALUE         => $req->datas->{_authChoice},
+                    CHOICE_VALUE         => $req->data->{_authChoice},
                     DISPLAY_FORM         => 0,
                     DISPLAY_OPENID_FORM  => 0,
                     DISPLAY_YUBIKEY_FORM => 0,
@@ -272,20 +387,24 @@ sub display {
             # Choose what form to display if not in a loop
             else {
 
-                my $displayType = $self->_authentication->getDisplayType($req);
+                my $displayType =
+                  eval { $self->_authentication->getDisplayType($req) };
 
                 $self->logger->debug("Display type $displayType ");
 
                 %templateParams = (
                     %templateParams,
-                    DISPLAY_FORM => $displayType =~ /\bstandardform\b/ ? 1 : 0,
+                    DISPLAY_FORM => $displayType =~ /\bstandardform\b/ ? 1
+                    : 0,
                     DISPLAY_OPENID_FORM => $displayType =~ /\bopenidform\b/ ? 1
                     : 0,
                     DISPLAY_YUBIKEY_FORM => $displayType =~ /\byubikeyform\b/
                     ? 1
                     : 0,
-                    DISPLAY_LOGO_FORM => $displayType eq "logo" ? 1 : 0,
-                    module => $displayType eq "logo"
+                    DISPLAY_SSL_FORM  => $displayType =~ /sslform/ ? 1 : 0,
+                    DISPLAY_GPG_FORM  => $displayType =~ /gpgform/ ? 1 : 0,
+                    DISPLAY_LOGO_FORM => $displayType eq "logo"    ? 1 : 0,
+                    module            => $displayType eq "logo"
                     ? $self->getModule( $req, 'auth' )
                     : "",
                     AUTH_LOOP => [],
@@ -300,23 +419,8 @@ sub display {
 
     }
 
-    ## Common template params
-    my $skin       = $self->getSkin($req);
-    my $portalPath = $self->conf->{portal};
-    $portalPath =~ s#^https?://[^/]+/?#/#;
-    $portalPath =~ s#[^/]+\.fcgi$##;
-    %templateParams = (
-        %templateParams,
-        SKIN_PATH => $portalPath . "skins",
-        ANTIFRAME => $self->conf->{portalAntiFrame},
-        SKIN_BG   => $self->conf->{portalSkinBackground},
-        %{ $self->customParameters },
-        %{ $req->{customParameters} // {} },
-    );
-
-    ## Custom template params
-    if ( my $customParams = $self->getCustomTemplateParameters() ) {
-        %templateParams = ( %templateParams, %$customParams );
+    if ( $req->data->{waitingMessage} ) {
+        $templateParams{WAITING_MESSAGE} = 1;
     }
 
     $self->logger->debug("Skin returned: $skinfile");
@@ -334,7 +438,8 @@ sub staticFile {
     require Cwd;
     require HTTP::Date;
     open my $fh, '<:raw', $self->conf->{templatesDir} . "/$file"
-      or return $self->sendError( $req, $!, 403 );
+      or return $self->sendError( $req,
+        $self->conf->{templatesDir} . "/$file: $!", 403 );
     my @stat = stat $file;
     Plack::Util::set_io_path( $fh, Cwd::realpath($file) );
     return [
@@ -360,7 +465,9 @@ sub buildHiddenForm {
           if $self->checkXSSAttack( $_, $req->{portalHiddenFormValues}->{$_} );
 
         # Build hidden input HTML code
-        $val .= qq{<input type="hidden" name="$_" id="$_" value="}
+        # 'id' is removed to avoid warning with Choice
+        #$val .= qq{<input type="hidden" name="$_" id="$_" value="}
+        $val .= qq{<input type="hidden" name="$_" value="}
           . $req->{portalHiddenFormValues}->{$_} . '" />';
     }
 
@@ -380,45 +487,40 @@ sub getSkin {
     $req->{sessionInfo}->{ipAddr} ||= $req->address;
 
     # Load specific skin from skinRules
-    foreach my $rule ( @{ $self->conf->{skinRules} } ) {
-        if ( $rule->[1]->( $req->sessionInfo ) ) {
-            $skin = $rule->[0];
-            $self->logger->debug("Skin $skin selected from skin rule");
+    foreach my $rule ( @{ $self->{skinRules} } ) {
+        if ( $rule->[1]->( $req, $req->sessionInfo ) ) {
+            if ( -d $self->conf->{templateDir} . '/' . $rule->[0] ) {
+                $skin = $rule->[0];
+                $self->logger->debug("Skin $skin selected from skin rule");
+                last;
+            }
         }
     }
 
     # Check skin GET/POST parameter
     my $skinParam = $req->param('skin');
-    if ( defined $skinParam && !$self->checkXSSAttack( 'skin', $skinParam ) ) {
-        $skin = $skinParam;
-        $self->logger->debug("Skin $skin selected from GET/POST parameter");
+    if ( defined $skinParam ) {
+        if ( $skinParam =~ /^[\w\-]+$/ ) {
+            if ( -d $self->conf->{templateDir} . '/' . $skinParam ) {
+                $skin = $skinParam;
+                $self->logger->debug(
+                    "Skin $skin selected from GET/POST parameter");
+            }
+            else {
+                $self->userLogger->error(
+                    "User tries to access to unexistent skin dir $skinParam");
+            }
+        }
+        else {
+            $self->userLogger->error("Strange skin parameter: $skinParam");
+        }
     }
 
     return $skin;
 }
 
-# Find custom templates parameters
-# @return Custom parameters
-sub getCustomTemplateParameters {
-    my ($self)          = @_;
-    my $conf            = $self->conf;
-    my $customTplParams = {};
-
-    foreach ( keys %$conf ) {
-        next unless ( $_ =~ /^tpl_(.+)$/ );
-        my $tplParam = $1;
-        my $tplValue = $conf->{$_};
-        $self->logger->debug(
-            "Set custom template parameter $tplParam with $tplValue");
-
-        $customTplParams->{$tplParam} = $tplValue;
-    }
-
-    return $customTplParams;
-}
-
 # Build an HTML array to display sessions
-# @param $sessions Array ref of hash ref containing sessions datas
+# @param $sessions Array ref of hash ref containing sessions data
 # @param $title Title of the array
 # @param $displayUser To display "User" column
 # @param $displaError To display "Error" column
@@ -428,29 +530,92 @@ sub mkSessionArray {
 
     return "" unless ( ref $sessions eq "ARRAY" and @$sessions );
 
-    my $tmp = $title ? "<h3>$title</h3>" : "";
-    $tmp .=
-        '<table class="info"><tbody><tr>'
-      . ( $displayUser ? '<th trspan="user">User</th>' : '' )
-      . '<th trspan="date">Date</th><th trspan="ipAddr">IP address</th>';
-    $tmp .= "<th>" . $self->conf->{sessionDataToRemember}->{$_} . "</th>"
-      foreach ( keys %{ $self->conf->{sessionDataToRemember} } );
-    $tmp .= '<th trspan="errorMsg">Error message</th>'
-      if ($displayError);
-    $tmp .= '</tr>';
+    my @fields = sort keys %{ $self->conf->{sessionDataToRemember} };
+    return $self->loadTemplate(
+        'sessionArray',
+        params => {
+            title        => $title,
+            displayUser  => $displayUser,
+            displayError => $displayError,
+            fields       => [
+                map { { name => $self->conf->{sessionDataToRemember}->{$_} } }
+                  @fields
+            ],
+            sessions => [
+                map {
+                    my $session = $_;
+                    {
+                        user   => $session->{user},
+                        utime  => $session->{_utime},
+                        ip     => $session->{ipAddr},
+                        values => [ map { { v => $session->{$_} } } @fields ],
+                        error  => $session->{error},
+                        displayError => $displayError,
+                    }
+                } @$sessions
+            ],
+        }
+    );
+}
 
-    foreach my $session (@$sessions) {
-        $tmp .= "<tr>"
-          . ( $displayUser ? "<td>$session->{user}</td>" : '' )
-          . qq'<td class="localeDate" val="$session->{_utime}"></td>'
-          . "<td>$session->{ipAddr}</td>";
-        $tmp .= "<td>" . ( $session->{$_} || "" ) . "</td>"
-          foreach ( keys %{ $self->conf->{sessionDataToRemember} } );
-        $tmp .= "<td>$session->{error}</td>" if ($displayError);
-        $tmp .= "</tr>";
+sub mkOidcConsent {
+    my ( $self, $session ) = @_;
+
+    if ( defined( $self->conf->{oidcRPMetaDataOptions} )
+        and ref( $self->conf->{oidcRPMetaDataOptions} ) )
+    {
+
+        # Set default RP displayname
+        foreach my $oidc ( keys %{ $self->conf->{oidcRPMetaDataOptions} } ) {
+            $self->conf->{oidcRPMetaDataOptions}->{$oidc}
+              ->{oidcRPMetaDataOptionsDisplayName} ||= $oidc;
+        }
     }
-    $tmp .= '</tbody></table>';
-    return $tmp;
+
+    # Loading existing oidcConsents
+    $self->logger->debug("Searching OIDC Consents...");
+    my $_oidcConsents;
+    if ( exists $session->{_oidcConsents} ) {
+        $_oidcConsents = eval {
+            from_json( $session->{_oidcConsents}, { allow_nonref => 1 } );
+        };
+        if ($@) {
+            $self->logger->error("Corrupted session (_oidcConsents): $@");
+            return PE_ERROR;
+        }
+    }
+    else {
+        $self->logger->debug("No OIDC Consent found");
+    }
+    my $consents = {};
+    foreach (@$_oidcConsents) {
+        if ( defined $_->{rp} ) {
+            my $rp = $_->{rp};
+            $self->logger->debug("RP { $rp } Consent found");
+            $consents->{$rp}->{epoch} = $_->{epoch};
+            $consents->{$rp}->{scope} = $_->{scope};
+            $consents->{$rp}->{displayName} =
+              $self->conf->{oidcRPMetaDataOptions}->{$rp}
+              ->{oidcRPMetaDataOptionsDisplayName};
+        }
+    }
+
+    # Display existing oidcConsents
+    return $self->loadTemplate(
+        'oidcConsents',
+        params => {
+            partners => [
+                map { {
+                        name        => $_,
+                        epoch       => $consents->{$_}->{epoch},
+                        scope       => $consents->{$_}->{scope},
+                        displayName => $consents->{$_}->{displayName}
+                    }
+                } ( sort keys %$consents )
+            ],
+            consents => join( ",", keys %$consents ),
+        }
+    );
 }
 
 1;
