@@ -9,11 +9,18 @@ package Lemonldap::NG::Portal::Main::Issuer;
 
 use strict;
 use Mouse;
-use Lemonldap::NG::Portal::Main::Constants qw(PE_OK);
+use MIME::Base64;
+use IO::String;
+use URI::Escape;
+use Lemonldap::NG::Common::FormEncode;
+use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
+  PE_RENEWSESSION
+);
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 # PROPERTIES
 
@@ -21,6 +28,18 @@ has type => ( is => 'rw' );
 
 has path => ( is => 'rw' );
 
+has ipath => ( is => 'rw' );
+
+has _ott => (
+    is      => 'rw',
+    lazy    => 1,
+    default => sub {
+        my $ott = $_[0]->{p}->loadModule('::Lib::OneTimeToken');
+        $ott->timeout( $_[0]->{conf}->{formTimeout} );
+        return $ott;
+    }
+);
+
 # INTERFACE
 
 # Only logout is called in normal use. Issuer that inherits from this
@@ -49,6 +68,7 @@ sub init {
     else {
         $self->logger->debug("No path declared for issuer $type. Skipping");
     }
+    $self->ipath( 'issuerRequest' . $self->path );
 }
 
 # RUNNING METHODS
@@ -57,18 +77,20 @@ sub init {
 
 sub _redirect {
     my ( $self, $req, @path ) = @_;
-    $self->logger->debug('Processing _redirect');
-    my $prms = $req->parameters;
-    foreach my $k ( keys %$prms ) {
-        $self->p->setHiddenFormValue( $req, $k, $prms->{$k}, '', 0 );
+    my $restore;
+    my $ir;
+    unless ( $self->can('ssoMatch') and not $self->ssoMatch($req) ) {
+        $self->logger->debug("Unauth request to $self->{path} issuer");
+        $restore = 1;
+        $self->logger->debug('Processing _redirect');
+        $ir = $req->pdata->{ $self->ipath } ||= $self->storeRequest($req);
+        $req->pdata->{ $self->ipath . 'Path' } = \@path;
+        $req->pdata->{keepPdata} = 1;
+        $req->{urldc} = $self->conf->{portal} . '/' . $self->path;
+    }
+    else {
+        $self->logger->debug('Not seen as Issuer request, skipping');
     }
-    $self->p->setHiddenFormValue( $req, 'issuerMethod', $req->method, '', 0 );
-    $self->p->setHiddenFormValue( $req, 'issuerQuery', $req->query_string, '',
-        0 );
-    $req->{urldc} =
-        $self->conf->{portal}
-      . $req->path
-      . ( $req->query_string ? '?' . $req->query_string : '' );
 
     # TODO: launch normal process with 'run' at the end
     return $self->p->do(
@@ -77,12 +99,23 @@ sub _redirect {
             'controlUrl',
             @{ $self->p->beforeAuth },
             $self->p->authProcess,
-            @{ $self->p->betweenAuthAndDatas },
-            $self->p->sessionDatas,
-            @{ $self->p->afterDatas },
-            sub {
-                return $self->run( @_, @path );
-              }
+            @{ $self->p->betweenAuthAndData },
+            $self->p->sessionData,
+            @{ $self->p->afterData },
+            $self->p->validSession,
+            @{ $self->p->endAuth },
+            (
+                $restore
+                ? sub {
+
+                    # Restore urldc if auth doesn't need to dial with browser
+                    $self->restoreRequest( $req, $ir );
+                    delete $req->pdata->{ $self->ipath };
+                    delete $req->pdata->{ $self->ipath . 'Path' };
+                    return $self->run( @_, @path );
+                  }
+                : ()
+            )
         ]
     );
 }
@@ -90,11 +123,24 @@ sub _redirect {
 # Case 3: authentified user, launch
 sub _forAuthUser {
     my ( $self, $req, @path ) = @_;
+
     $self->logger->debug('Processing _forAuthUser');
+    if ( my $r = $req->pdata->{ $self->ipath } ) {
+        $self->logger->debug("Restoring request to $self->{path} issuer");
+        $self->restoreRequest( $req, $r );
+        @path = @{ $req->pdata->{ $self->ipath . 'Path' } }
+          if ( $req->pdata->{ $self->ipath . 'Path' } );
+    }
+
+    # Clean pdata: keepPdata has been set, so pdata must be cleaned here
+    $self->logger->debug('Cleaning pdata');
+    $req->pdata( {} );
+    $req->urlNotBase64(1) if ( ref($self) =~ /::CAS$/ );
+    $req->mustRedirect(1);
     return $self->p->do(
         $req,
         [
-            'importHandlerDatas',
+            'importHandlerData',
             'controlUrl',
             @{ $self->p->forAuthUser },
             sub {
@@ -104,6 +150,50 @@ sub _forAuthUser {
     );
 }
 
+sub storeRequest {
+    my ( $self, $req ) = @_;
+    $self->logger->debug('Store issuer request');
+    my $info = {};
+    $info->{content} = $req->content;
+    foreach ( keys %{ $req->env } ) {
+        $info->{$_} = $req->env->{$_} unless ( ref $req->env->{$_} );
+    }
+    return $self->_ott->createToken($info);
+}
+
+sub restoreRequest {
+    my ( $self, $req, $token ) = @_;
+    my $env = $self->_ott->getToken($token);
+    if ($env) {
+        $self->logger->debug("Restoring request from $token");
+        if ( my $c = delete $env->{content} ) {
+            $env->{'psgix.input.buffered'} = 0;
+            $env->{'psgi.input'}           = IO::String->new($c);
+        }
+        $req->{env} = {};
+        foreach ( keys %$env ) {
+            $self->logger->debug(
+                "Restore $_" . ( ref $env->{$_} ? '' : "\t" . $env->{$_} ) );
+            $req->env->{$_} = $env->{$_} unless /^plack/;
+        }
+    }
+    $req->{uri} = uri_unescape( $req->env->{REQUEST_URI} );
+    $req->{uri} =~ s|^//+|/|g;
+    return $req;
+}
+
+sub reAuth {
+    my ( $self, $req ) = @_;
+    $req->data->{customScript} =
+qq'<script type="text/javascript" src="$self->{p}->{staticPrefix}/common/js/autoRenew.min.js"></script>'
+      if ( $self->conf->{skipRenewConfirmation} );
+    $req->data->{_url} =
+      encode_base64( $self->conf->{portal} . $req->path_info, '' );
+    $req->pdata->{ $self->ipath } = $self->storeRequest($req);
+    $req->pdata->{keepPdata} = 1;
+    return PE_RENEWSESSION;
+}
+
 1;
 __END__
 
@@ -122,6 +212,9 @@ Lemonldap::NG::Portal::Main::Issuer - Base class for identity providers.
   use Mouse;
   extends 'Lemonldap::NG::Portal::Main::Issuer';
   use Lemonldap::NG::Portal::Main::Constants qw(PE_OK);
+
+  # Required: URL root path
+  use constant path => 'saml';
   
   # Optional initialization method
   sub init {
@@ -185,6 +278,10 @@ URI path:
 They must be called during initialization process (so you must write the
 optional init() sub).
 
+Be careful with C<add*authRoute()>: you can't catch here your root path (=
+path declared in C<$self-E<gt>path>) because it is caught by this module,
+but you can catch sub-routes (ie C</path/something>).
+
 Example:
 
   sub init {
@@ -200,6 +297,13 @@ Example:
       return [ 200, [ 'Content-Type' => 'application/xml' ], [] ];
   }
 
+=head2 avoid conflicts in path
+
+If you share base URI path with another plugin (a C<Auth::*> module for
+example), it is recommended to write a C<ssoMatch> function that returns true
+if C<$req-E<gt>uri> has to be handled by Issuer module. See C<Issuer::SAML>
+or C<Issuer::OpenIDConnect> to have some examples.
+
 =head1 SEE ALSO
 
 L<http://lemonldap-ng.org/>
@@ -215,7 +319,7 @@ L<http://lemonldap-ng.org/>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 =head1 DOWNLOAD
 
