@@ -7,7 +7,7 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants ':all';
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.3';
 
 extends 'Lemonldap::NG::Common::PSGI::Request';
 
@@ -104,6 +104,7 @@ sub error_type {
                 PE_CONFIRM,                       PE_MAILFORMEMPTY,
                 PE_MAILCONFIRMATION_ALREADY_SENT, PE_PASSWORDFORMEMPTY,
                 PE_CAPTCHAEMPTY,                  PE_REGISTERFORMEMPTY,
+                PE_PP_CHANGE_AFTER_RESET
             )
         )
       );
@@ -115,9 +116,12 @@ sub error_type {
 }
 
 sub init {
-    my ($self) = @_;
+    my ( $self, $conf ) = @_;
     $self->{$_} = {} foreach (qw(data customParameters sessionInfo pdata));
     $self->{$_} = [] foreach (qw(respCookies));
+    if ( my $tmp = $self->userData->{ $conf->{whatToTrace} } ) {
+        $self->user($tmp);
+    }
 }
 
 sub errorString {
