@@ -22,10 +22,10 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_TOKENEXPIRED
 );
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.3';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin',
-  'Lemonldap::NG::Portal::Lib::SMTP';
+  'Lemonldap::NG::Portal::Lib::SMTP', 'Lemonldap::NG::Portal::Lib::_tokenRule';
 
 # PROPERTIES
 
@@ -77,7 +77,7 @@ sub init {
     }
 
     # Initialize form token if needed (captcha provides also a token)
-    elsif ( $self->conf->{requireToken} ) {
+    else {
         $_[0]->ott(
             $_[0]->p->loadModule('Lemonldap::NG::Portal::Lib::OneTimeToken') )
           or return 0;
@@ -165,9 +165,10 @@ sub _register {
             and
             !$self->getRegisterSession( $req->data->{registerInfo}->{mail} ) )
         {
+            
             # Check if token exists
             my $token;
-            if ( $self->ott or $self->captcha ) {
+            if ( $self->ottRule->( $req, {} ) or $self->captcha ) {
                 $token = $req->param('token');
                 unless ($token) {
                     $self->setSecurity($req);
@@ -199,7 +200,7 @@ sub _register {
                 }
                 $self->logger->debug("Captcha code verified");
             }
-            elsif ( $self->ott ) {
+            elsif ( $self->ottRule->( $req, {} ) ) {
                 unless ( $self->ott->getToken($token) ) {
                     $self->setSecurity($req);
                     $self->userLogger->notice(
@@ -220,7 +221,7 @@ sub _register {
     $req->user( $req->data->{registerInfo}->{mail} );
     if ( $self->p->_userDB->getUser( $req, useMail => 1 ) == PE_OK ) {
         $self->userLogger->error(
-"Register: refuse mail $req->{mail} because already exists in UserDB"
+"Register: refuse mail $req->{data}->{registerInfo}->{mail} because already exists in UserDB"
         );
         return PE_REGISTERALREADYEXISTS;
     }
@@ -233,8 +234,7 @@ sub _register {
     unless ( $req->data->{register_token} or $register_session ) {
 
         # Create mail token
-        $register_session = $self->mailott->createToken(
-            {
+        $register_session = $self->mailott->createToken( {
                 mail      => $req->data->{registerInfo}->{mail},
                 firstname => $req->data->{registerInfo}->{firstname},
                 lastname  => $req->data->{registerInfo}->{lastname},
@@ -400,7 +400,7 @@ sub _register {
 sub display {
     my ( $self, $req ) = @_;
     my %templateParams = (
-        SKIN_PATH       => '/static',
+        SKIN_PATH       => $self->conf->{staticPrefix},
         SKIN            => $self->p->getSkin($req),
         SKIN_BG         => $self->conf->{portalSkinBackground},
         MAIN_LOGO       => $self->conf->{portalMainLogo},
@@ -430,11 +430,9 @@ sub display {
     );
 
     # Display form the first time
-    if (
-        (
+    if ( (
                $req->error == PE_REGISTERFORMEMPTY
             or $req->error == PE_REGISTERFIRSTACCESS
-            or $req->error == PE_REGISTERALREADYEXISTS
             or $req->error == PE_CAPTCHAERROR
             or $req->error == PE_CAPTCHAEMPTY
             or $req->error == PE_NOTOKEN
@@ -462,6 +460,17 @@ sub display {
         $templateParams{TOKEN} = $req->token;
     }
 
+    if ( $req->error == PE_REGISTERALREADYEXISTS ) {
+        %templateParams = (
+            %templateParams,
+            DISPLAY_FORM            => 0,
+            DISPLAY_RESEND_FORM     => 0,
+            DISPLAY_CONFIRMMAILSENT => 0,
+            DISPLAY_MAILSENT        => 0,
+            DISPLAY_PASSWORD_FORM   => 0,
+        );
+    }
+
     # Display mail confirmation resent form
     if ( $req->{error} == PE_MAILCONFIRMATION_ALREADY_SENT ) {
         %templateParams = (
@@ -513,6 +522,7 @@ sub display {
             DISPLAY_PASSWORD_FORM   => 1,
         );
     }
+
     return ( 'register', \%templateParams );
 }
 
@@ -521,7 +531,7 @@ sub setSecurity {
     if ( $self->captcha ) {
         $self->captcha->setCaptcha($req);
     }
-    elsif ( $self->ott ) {
+    elsif ( $self->ottRule->( $req, {} ) ) {
         $self->ott->setToken($req);
     }
 }
