@@ -125,8 +125,7 @@ sub clean_sessions {
 
 sub getCache {
     require Cache::FileCache;
-    return Cache::FileCache->new(
-        {
+    return Cache::FileCache->new( {
             namespace   => 'lemonldap-ng-session',
             cache_root  => 't/',
             cache_depth => 0,
@@ -247,14 +246,32 @@ m@<form.+?action="(?:(?:http://([^/]+))?(/.*?)?|(#))".+method="(post|get)"@is,
             }
             count(1);
         }
+
+        # Fields with values
         my %fields =
           ( $res->[2]->[0] =~
-              m#<input.+?name="([^"]+)"[^>]+?value="([^"]*?)"#gs );
-        my $query = join( '&',
-            map { "$_=" . uri_escape( uri_unescape( $fields{$_} ) ) }
-              keys(%fields) );
+              m#<input.+?name="([^"]+)"[^>]+(?:value="([^"]*?)")#gs );
+
+        # Add fields without values
+        %fields = (
+            $res->[2]->[0] =~
+              m#<input.+?name="([^"]+)"[^>]+(?:value="([^"]*?)")?#gs,
+            %fields
+        );
+        my $query = join(
+            '&',
+            map {
+                "$_="
+                  . (
+                    $fields{$_}
+                    ? uri_escape( uri_unescape( $fields{$_} ) )
+                    : ''
+                  )
+              }
+              keys(%fields)
+        );
         foreach my $f (@requiredFields) {
-            ok( defined $fields{$f}, qq{ Field "$f" is defined} );
+            ok( exists $fields{$f}, qq{ Field "$f" is defined} );
             count(1);
         }
         exceptCspFormOK( $res, $host );
@@ -606,8 +623,7 @@ to test content I<(to launch a C<expectForm()> for example)>.
 
 sub _get {
     my ( $self, $path, %args ) = @_;
-    my $res = $self->app->(
-        {
+    my $res = $self->app->( {
             'HTTP_ACCEPT' => $args{accept}
               || 'application/json, text/plain, */*',
             'HTTP_ACCEPT_LANGUAGE' => 'fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3',
@@ -644,7 +660,7 @@ sub _get {
 Same as C<_get> except that a body is required. $body must be a file handle.
 Example with IO::String:
 
-  ok( 
+  ok(
       $res = $client->_post(
           '/',
           IO::String->new('user=dwho&password=dwho'),
@@ -659,8 +675,7 @@ sub _post {
     my ( $self, $path, $body, %args ) = @_;
     die "$body must be a IO::Handle"
       unless ( ref($body) and $body->can('read') );
-    my $res = $self->app->(
-        {
+    my $res = $self->app->( {
             'HTTP_ACCEPT' => $args{accept}
               || 'application/json, text/plain, */*',
             'HTTP_ACCEPT_LANGUAGE' => 'fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3',
